@@ -27,7 +27,7 @@ import os
 import inspect
 import warnings
 import datetime
-from typing import Optional, Union
+from typing import Optional
 
 
 class TestProjectLibrary:
@@ -44,7 +44,8 @@ class TestProjectLibrary:
         self.ROBOT_LIBRARY_LISTENER = self
         self.__default_screenshot_name = f'TestProject-{datetime.date.today().strftime("%Y_%m_%d_%H_%M_%S")}.png'
         self.__reporter = None
-        self.library = SeleniumLibrary()
+        self.__library = SeleniumLibrary()
+        self.__is_generic = False
         os.environ["RFW_SUPPRESS_WARNINGS"] = "true"
 
     # TESTPROJECT #
@@ -56,7 +57,7 @@ class TestProjectLibrary:
         timeout: Optional[int] = 5000,
         project_name: Optional[str] = "TestProject Robot",
         job_name: Optional[str] = "Robot Job",
-        desired_capabilities: Union[str, dict, None] = None,
+        desired_capabilities=None,
         disabled_reports: Optional[bool] = False,
         dev_token: Optional[str] = os.getenv("TP_DEV_TOKEN"),
     ):
@@ -93,52 +94,82 @@ class TestProjectLibrary:
             driver = webdriver.Firefox(
                 firefox_options=desired_capabilities,
                 token=dev_token,
-                projectname=project_name,
-                jobname=job_name,
-                disable_reports=disabled_reports)
+                project_name=project_name,
+                job_name=job_name,
+                disable_reports=disabled_reports,
+            )
         elif browser in self.CHROME_NAMES:
             driver = webdriver.Chrome(
                 chrome_options=desired_capabilities,
                 token=dev_token,
-                projectname=project_name,
-                jobname=job_name,
-                disable_reports=disabled_reports)
+                project_name=project_name,
+                job_name=job_name,
+                disable_reports=disabled_reports,
+            )
         elif browser in self.IE_NAMES:
             driver = webdriver.Ie(
                 ie_options=desired_capabilities,
                 token=dev_token,
-                projectname=project_name,
-                jobname=job_name,
-                disable_reports=disabled_reports)
+                project_name=project_name,
+                job_name=job_name,
+                disable_reports=disabled_reports,
+            )
         elif browser == "edge":
             driver = webdriver.Edge(
                 edge_options=desired_capabilities,
                 token=dev_token,
-                projectname=project_name,
-                jobname=job_name,
-                disable_reports=disabled_reports)
+                project_name=project_name,
+                job_name=job_name,
+                disable_reports=disabled_reports,
+            )
         elif browser == "safari":
             driver = webdriver.Safari(
                 desired_capabilities=desired_capabilities,
                 token=dev_token,
-                projectname=project_name,
-                jobname=job_name,
-                disable_reports=disabled_reports)
+                project_name=project_name,
+                job_name=job_name,
+                disable_reports=disabled_reports,
+            )
+        elif browser == "generic":
+            driver = webdriver.Generic(
+                token=dev_token,
+                project_name=project_name,
+                job_name=job_name,
+                disable_reports=disabled_reports
+            )
+            self.__is_generic = True
         else:
             raise ValueError("Unsupported Browser, please look at the official TestProject library documentation")
 
-        driver.report().disable_command_reports(True)
-        driver.set_script_timeout(timeout)
-        driver.report().step(message="Set timeout", description=f"Time out was set to {timeout}", passed=True)
-        try:
-            driver.get(url)
-            driver.report().step(message=f"Navigated to {url}", description=f"Successfully navigated to {url}", passed=True)
-        except Exception:
-            driver.report().step(message=f"Failed to open {url}", description=f"Failed to open {url}", passed=True)
-            raise
-        self.library.register_driver(driver=driver, alias="testproject_driver")
+        # Set browser only if the driver is not generic
+        if not self.__is_generic:
+            driver.report().disable_command_reports(True)
+            driver.report().step(
+                message="Set timeout",
+                description=f"Time out was set to {timeout} millisecond",
+                passed=True
+            )
+            try:
+                driver.get(url)
+                driver.report().step(
+                    message=f"Navigated to {url}",
+                    description=f"Successfully navigated to {url}",
+                    passed=True
+                )
+            except Exception:
+                driver.report().step(message=f"Failed to open {url}", description=f"Failed to open {url}", passed=True)
+                raise
+            self.__library.register_driver(driver=driver, alias="testproject_driver")
+        else:
+            driver.report().step(
+                message="Generic Driver",
+                description="New session created",
+                passed=True,
+                screenshot=False
+            )
+
         self.__reporter = driver.report()
-        self.__reporter.exclude_test_names(['run_cli', 'main'])
+        self.__reporter.exclude_test_names(["run_cli", "main"])
 
     def _build_capabilities(self, caps, browser_name):
         options = None
@@ -604,63 +635,76 @@ class TestProjectLibrary:
     # WAITING #
     @keyword
     def wait_for_condition(self, condition, timeout=None, error=None):
-        self.base("", f"Condition: {condition} was met under {timeout} seconds", f"{condition}", condition, timeout, error)
+        message = self._set_message(timeout)
+        self.base("", f"Condition: '{condition}' was met {message}", f"{condition}", condition, timeout, error)
 
     @keyword
     def wait_until_location_is(self, expected, timeout=None, message=None):
-        self.base("", f"Location was {expected} under {timeout} seconds", f"{expected}", expected, timeout, message)
+        message = self._set_message(timeout)
+        self.base("", f"Location was '{expected}' {message}", f"{expected}", expected, timeout, message)
 
     @keyword
     def wait_until_location_is_not(self, location, timeout=None, message=None):
-        self.base("", f"Location was not {location} under {timeout} seconds", f"{location}", location, timeout, message)
+        message = self._set_message(timeout)
+        self.base("", f"Location was not '{location}' {message}", f"{location}", location, timeout, message)
 
     @keyword
     def wait_until_location_contains(self, expected, timeout=None, message=None):
-        self.base("", f"Location contained {expected} under {timeout} seconds", f"{expected}", expected, timeout, message)
+        message = self._set_message(timeout)
+        self.base("", f"Location contained '{expected}' {message}", f"{expected}", expected, timeout, message)
 
     @keyword
     def wait_until_location_does_not_contain(self, location, timeout=None, message=None):
-        self.base(
-            "", f"Location does not contain {location} under {timeout} seconds", f"{location}", location, timeout, message
-        )
+        message = self._set_message(timeout)
+        self.base("", f"Location does not contain '{location}' {message}", f"{location}", location, timeout, message)
 
     @keyword
     def wait_until_page_contains(self, text, timeout=None, error=None):
-        self.base("", f"Page contained {text} under {timeout} seconds", f" {text}", text, timeout, error)
+        message = self._set_message(timeout)
+        self.base("", f"Page contained '{text}' {message}", f" {text}", text, timeout, error)
 
     @keyword
     def wait_until_page_does_not_contain(self, text, timeout=None, error=None):
-        self.base("", f"Page does not contain {text} under {timeout} seconds", f" {text}", text, timeout, error)
+        message = self._set_message(timeout)
+        self.base("", f"Page does not contain '{text}' {message}", f" {text}", text, timeout, error)
 
     @keyword
     def wait_until_page_contains_element(self, locator, timeout=None, error=None, limit=None):
-        self.base(locator, f"Page contained {locator} under {timeout} seconds", f" {locator}", timeout, error, limit)
+        message = self._set_message(timeout)
+        self.base(locator, f"Page contained '{locator}' {message}", f" {locator}", timeout, error, limit)
 
     @keyword
     def wait_until_page_does_not_contain_element(self, locator, timeout=None, error=None, limit=None):
-        self.base(locator, f"Page does not contain {locator} under {timeout} seconds", f" {locator}", timeout, error, limit)
+        message = self._set_message(timeout)
+        self.base(locator, f"Page does not contain '{locator}' {message}", f" {locator}", timeout, error, limit)
 
     @keyword
     def wait_until_element_is_visible(self, locator, timeout=None, error=None):
-        self.base(locator, f"Element {locator} was visible under {timeout} seconds", f" {locator}", timeout, error)
+        message = self._set_message(timeout)
+        self.base(locator, f"Element '{locator}' was visible {message}", f" {locator}", timeout, error)
 
     @keyword
     def wait_until_element_is_not_visible(self, locator, timeout=None, error=None):
-        self.base(locator, f"Element {locator} was not visible under {timeout} seconds", f" {locator}", timeout, error)
+        message = self._set_message(timeout)
+        self.base(locator, f"Element '{locator}' was not visible {message}", f" {locator}", timeout, error)
 
     @keyword
     def wait_until_element_is_enabled(self, locator, timeout=None, error=None):
-        self.base(locator, f"Element {locator} was enabled under {timeout} seconds", f" {locator}", timeout, error)
+        message = self._set_message(timeout)
+        self.base(locator, f"Element '{locator}' was enabled {message}", f" {locator}", timeout, error)
 
     @keyword
     def wait_until_element_contains(self, locator, text, timeout=None, error=None):
-        self.base(locator, f"Element {locator} contained {text} under {timeout} seconds", f" {text}", text, timeout, error)
+        message = self._set_message(timeout)
+        self.base(locator, f"Element '{locator}' contained {text} {message}", f" {text}", text, timeout, error)
 
     @keyword
     def wait_until_element_does_not_contain(self, locator, text, timeout=None, error=None):
-        self.base(
-            locator, f"Element {locator} does not contain {text} under {timeout} seconds", f" {text}", text, timeout, error
-        )
+        message = self._set_message(timeout)
+        self.base(locator, f"Element '{locator}' does not contain {text} {message}", f" {text}", text, timeout, error)
+
+    def _set_message(self, timeout):
+        return "" if timeout is None else f"(timeout: {timeout} seconds)"
 
     # WAITING END #
 
@@ -933,15 +977,15 @@ class TestProjectLibrary:
 
     @keyword
     def set_selenium_speed(self, value):
-        return self.base("", f"Previous speed\nNew speed set to {value}", f"{value}", value)
+        return self.base("", f"Selenium Speed set to {value}", f"{value}", value)
 
     @keyword
     def set_selenium_timeout(self, value):
-        return self.base("", f"Previous timeout\nNew timeout set to {value}", f"{value}", value)
+        return self.base("", f"Timeout was set to {value}", f"{value}", value)
 
     @keyword
     def set_selenium_implicit_wait(self, value):
-        return self.base("", f"Previous implicit wait\nNew implicit wait set to {value}", f"{value}", value)
+        return self.base("", f"Implicit wait set to {value}", f"{value}", value)
 
     @keyword
     def set_browser_implicit_wait(self, value):
@@ -970,6 +1014,17 @@ class TestProjectLibrary:
 
     # BROWSER MANAGEMENT END #
 
+    # GENERIC #
+    @keyword
+    def create_step(self, description, message, passed=True, screenshot=False):
+        self.__reporter.step(
+            message=message,
+            description=description,
+            passed=passed,
+            screenshot=False if self.__is_generic else screenshot
+        )
+    # GENERIC END#
+
     # UTIL METHODS #
     def base(self, *args):
         func_name = inspect.stack()[1].function
@@ -985,7 +1040,7 @@ class TestProjectLibrary:
             if not value:
                 self.base_report(True, message=message, func_name=func_name, description=description)
             else:
-                self.base_report(True, message=f" {message}: '{value}'", func_name=func_name, description=description)
+                self.base_report(True, message=f"Returned value: {value}", func_name=func_name, description=description)
             return value
         except Exception as e:
             self.base_report(success=False, exception=e, func_name=func_name, description=description)
@@ -1011,7 +1066,7 @@ class TestProjectLibrary:
         func_name = self.convert(func_name)
         if success:
             self.__reporter.step(
-                description=f"{func_name}: {description}", message=message, passed=True, screenshot=screenshot
+                description=f"{func_name}: {description}", message=message, passed=True, screenshot=True
             )
         else:
             if not message:
@@ -1024,17 +1079,17 @@ class TestProjectLibrary:
             if not func_name:  # Get name of calling Keyword
                 func_name = inspect.stack()[1].function
             if not locator and not values:
-                ret_value = self.library.run_keyword(func_name, (), {})
+                ret_value = self.__library.run_keyword(func_name, (), {})
             elif not locator:
-                ret_value = self.library.run_keyword(func_name, self.build_values(None, *values), {})
+                ret_value = self.__library.run_keyword(func_name, self.build_values(None, *values), {})
             elif not values:
                 if func_name == "get_webelements":
                     func_name = "Get WebElements"
                 if func_name == "get_webelement":
                     func_name = "Get WebElement"
-                ret_value = self.library.run_keyword(func_name, [locator], {})
+                ret_value = self.__library.run_keyword(func_name, [locator], {})
             else:
-                ret_value = self.library.run_keyword(func_name, self.build_values(locator, *values), {})
+                ret_value = self.__library.run_keyword(func_name, self.build_values(locator, *values), {})
             return ret_value
         except Exception as e:
             raise e
